//===-- X86MASMPrinter.h - Convert X86 LLVM code to MASM assembly ---------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file was developed by the LLVM research group and is distributed under
// the University of Illinois Open Source License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// MASM Microsoft Assembler printer.
//
//===----------------------------------------------------------------------===//

#ifndef X86MASMPRINTER_H
#define X86MASMPRINTER_H

#include "X86.h"
#include "X86TargetMachine.h"
#include "X86AsmPrinter.h"
#include "llvm/CodeGen/ValueTypes.h"
#include "llvm/Target/TargetMachine.h"

namespace llvm {
namespace x86 {

struct X86MASMPrinter : public X86SharedAsmPrinter {
  X86MASMPrinter(std::ostream &o, TargetMachine &tm)
    : X86SharedAsmPrinter( o, tm)
      {
        CommentString = ";";
        GlobalPrefix = "_";
        ZeroDirective = 0;
        AsciiDirective = "\tBYTE\t";
        Data8bitsDirective = "\tBYTE\t";
        Data16bitsDirective = "\tWORD\t";
        Data32bitsDirective = "\tDWORD\t";
        Data64bitsDirective = "\tQWORD\t";
        AlignDirective ="\tALIGN\t";
      }

  virtual const char *getPassName() const {
    return "X86 MASM Assembly Printer";
  }

  /// printInstruction - This method is automatically generated by tablegen
  /// from the instruction set description.  This method returns true if the
  /// machine instruction was sufficiently described to print it, otherwise it
  /// returns false.
  bool printInstruction(const MachineInstr *MI);

  // This method is used by the tablegen'erated instruction printer.
  void printOperand(const MachineInstr *MI, unsigned OpNo, MVT::ValueType VT);
  void printCallOperand(const MachineInstr *MI, unsigned OpNo,
                        MVT::ValueType VT);
  void printMemoryOperand(const MachineInstr *MI, unsigned OpNo,
                          MVT::ValueType VT);
  void printMachineInstruction(const MachineInstr *MI);
  void printOp(const MachineOperand &MO, bool elideOffsetKeyword = false);
  void printMemReference(const MachineInstr *MI, unsigned Op);
  bool runOnMachineFunction(MachineFunction &F);

  bool doInitialization(Module &M);
  bool doFinalization(Module &M);

  void printExternals( Module &M);

  std::string getMASMTypeName(const Type *Ty);
};

} // end namespace x86
} // end namespace llvm

#endif
